package ca.canadapost.cpcdp.postoffice;

import ca.canadapost.cpcdp.postoffice.generated.messages.Messages;
import ca.canadapost.cpcdp.postoffice.generated.postoffice.*;

import com.sun.jersey.api.client.*;
import com.sun.jersey.api.client.config.*;

import javax.xml.bind.JAXBContext;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Properties;

/**
 * Sample code for the GetPostOfficeDetail Canada Post service.
 * <p>
 * The GetPostOfficeDetail service returns Post Office details such as the daily 
 * hours of operation for a given Post Office as determined by the first call.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 * <p>
 *  Jersey REST client using JAXB for GetPostOfficeDetail Request.  <br>
 *  USAGE:<pre>
 *        GetPostOfficeDetailClient client = new GetPostOfficeDetailClient(username, password);
 *        ClientResponse response = client.getPostOfficeDetail();
 *        // do whatever with response
 *        client.close();
 *  </pre>
 *
 */
public class GetPostOfficeDetailClient {

    private Client aClient;
    
    // REST URI
    private static final String LINK = "https://ct.soa-gw.canadapost.ca/rs/postoffice/102978/detail";
    
    // Main class for local testing only
    public static void main(String[] args) {

    	// Your username and password are imported from the following file
    	// CPCWS_PostOffice_Java_Samples/user.properties 
    	Properties userProps = new Properties();
    	FileInputStream propInputStream;
		try {
			propInputStream = new FileInputStream("user.properties");
			userProps.load(propInputStream);
			propInputStream.close(); // better in finally block
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.out);
			return;
		}
    	String username = userProps.getProperty("username");
    	String password = userProps.getProperty("password");
						
		// Execute GetPostOfficeDetail Request
        GetPostOfficeDetailClient myClient = new GetPostOfficeDetailClient(username, password);
        ClientResponse resp = myClient.getPostOfficeDetail();
        InputStream respIS = resp.getEntityInputStream();
        
        System.out.println("HTTP Response Status: " + resp.getStatus() + " " + resp.getClientResponseStatus());

        // Example of using JAXB to parse xml response
        JAXBContext jc;
        try {
			jc = JAXBContext.newInstance(PostOfficeDetail.class, Messages.class);
            Object entity = jc.createUnmarshaller().unmarshal(respIS);
            // Determine whether response data matches PostOffice schema.
            if (entity instanceof PostOfficeDetail) {            	
            	PostOfficeDetail postOffice = (PostOfficeDetail) entity;           	
            	System.out.println("Office Name: " + postOffice.getName());
                System.out.println("Address: " + postOffice.getAddress().getOfficeAddress());
                System.out.println();
            	for (Iterator<PostOfficeDetail.HoursList> iter = postOffice.getHoursLists().iterator(); iter.hasNext();) {
            		PostOfficeDetail.HoursList hoursListItem = (PostOfficeDetail.HoursList) iter.next();
            		System.out.println("Weekday: " + hoursListItem.getDay());
            		for (Iterator<String> iter2 = hoursListItem.getTimes().iterator(); iter2.hasNext();) {
                		String timeItem = (String) iter2.next();
                		System.out.println("Time: " + timeItem);
            		}
	                System.out.println();
                }
            } else {
                // Assume Error Schema
                Messages messageData = (Messages) entity;
                for (Iterator<Messages.Message> iter = messageData.getMessage().iterator(); iter.hasNext();) {
                    Messages.Message aMessage = (Messages.Message) iter.next();
                    System.out.println("Error Code: " + aMessage.getCode());
                    System.out.println("Error Msg: " + aMessage.getDescription());
                }
            }
        } catch (Exception e) {
        	e.printStackTrace(System.out);
        }

        myClient.close();
    }

    public GetPostOfficeDetailClient(String username, String password) {
        ClientConfig config = new DefaultClientConfig();
        aClient = Client.create(config);
        aClient.addFilter(new com.sun.jersey.api.client.filter.HTTPBasicAuthFilter(username, password));
    }

    public ClientResponse getPostOfficeDetail() throws UniformInterfaceException {    	
    	WebResource aWebResource = aClient.resource(LINK);
        return aWebResource.accept("application/vnd.cpc.postoffice+xml").acceptLanguage("en-CA").get(ClientResponse.class);
    }

    public void close() {
        aClient.destroy();
    }
}
